﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TickDataConvertor.Properties;
using System.IO;
using KreslikLib.Common;
using System.Diagnostics;

namespace TickDataConvertor
{
    public delegate void StatusBarChangedHandler(string NewText);

    public partial class MainWindow : Form
    {
        bool InputFolderSelected = false;
        bool OutputFolderSelected = false;
        bool Abort;
        string[] InputFiles;
        string OutputFolder;
        Settings AppSettings;

        protected event StatusBarChangedHandler StatusChanged;

        public MainWindow()
        {
            InitializeComponent();
            AppSettings = new Settings();

            if (AppSettings.LastInputFolder != "")
            {
                InputFiles = Directory.GetFiles(AppSettings.LastInputFolder);
                this.labelInputFolder.Text = AppSettings.LastInputFolder;
                InputFolderSelected = true;
            }

            if (AppSettings.LastOutputFolder != "")
            {
                OutputFolder = AppSettings.LastOutputFolder;
                this.labelOutputFolder.Text = AppSettings.LastOutputFolder;
                OutputFolderSelected = true;
            }

            this.Text += " ver. " + AppSettings.Version;
            this.checkBoxRemoveSame.Checked = AppSettings.RemoveSamePrice;
            this.StartDate.Value = AppSettings.StartDate;
            this.EndDate.Value = AppSettings.EndDate;
            ChangeDataSelectionVisuals();

            StatusChanged += new StatusBarChangedHandler(MainWindow_StatusChanged);
        }

        private void ChangeDataSelectionVisuals()
        {
            this.radioButtonAlldata.Enabled = true;
            this.radioButtonDataInRange.Enabled = true;

            if (AppSettings.AllData)
            {
                this.radioButtonAlldata.Checked = true;
                this.radioButtonDataInRange.Checked = false;
                this.labelStartDate.Enabled = false;
                this.labelEndDate.Enabled = false;
                this.StartDate.Enabled = false;
                this.EndDate.Enabled = false;
            }
            else
            {
                this.radioButtonAlldata.Checked = false;
                this.radioButtonDataInRange.Checked = true;
                this.labelStartDate.Enabled = true;
                this.labelEndDate.Enabled = true;
                this.StartDate.Enabled = true;
                this.EndDate.Enabled = true;
            }
        }

        private void MainWindow_StatusChanged(string NewText)
        {
            statusBar.Text = NewText;
        }

        private void StartConversion(object sender, EventArgs e)
        {
            if (!(InputFolderSelected && OutputFolderSelected))
            {
                KTools.InfoBox("Please select both the input and output folders first.", "Select folders first");
            }
            else
            {
                StartConversionVisuals();
                backgroundWorker1.RunWorkerAsync();
            }
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            PerformConversion(InputFiles, OutputFolder);
        }

        private void StartConversionVisuals()
        {

            this.Abort = false;
            this.progressBar.Visible = true;
            this.progressBar.Value = 0;
            this.buttonAbort.Visible = true;
            this.buttonAbort.Text = "Abort";
            this.buttonAbort.Enabled = true;
            this.buttonStartConv.Visible = false;


            this.button1.Enabled = false;
            this.buttonStartConv.Enabled = false;
            this.button3.Enabled = false;
            this.checkBoxRemoveSame.Enabled = false;

            this.radioButtonAlldata.Enabled = false;
            this.radioButtonDataInRange.Enabled = false;
            this.StartDate.Enabled = false;
            this.EndDate.Enabled = false;

            this.labelStartDate.Enabled = false;
            this.labelEndDate.Enabled = false;

            this.labelInputFolder.Enabled = false;
            this.labelOutputFolder.Enabled = false;

            this.buttonTSexport.Enabled = false;
        }

        private void EndConversionVisuals()
        {
            this.ChangeDataSelectionVisuals();
            this.button1.Enabled = true;
            this.buttonStartConv.Enabled = true;
            this.button3.Enabled = true;
            this.progressBar.Visible = false;
            this.buttonStartConv.Visible = true;
            this.buttonAbort.Visible = false;
            this.checkBoxRemoveSame.Enabled = true;
            this.statusBar.Text = "Ready.";
            this.labelInputFolder.Enabled = true;
            this.labelOutputFolder.Enabled = true;
            this.buttonTSexport.Enabled = true;
        }

        private void DisableAbortButton()
        {
            this.buttonAbort.Enabled = false;
        }

        private void SetProgressBar(int ProgressValue)
        {
            this.progressBar.Value = 100;
        }

        protected delegate void DisableAbortButtonDelegate();
        protected delegate void SetProgressBarDelegate(int ProgressValue);
        
        private void PerformConversion(string[] InputFiles, string OutputFolder)
        {
            DisableAbortButtonDelegate DisableAbort = new DisableAbortButtonDelegate(DisableAbortButton);
            SetProgressBarDelegate SetProgress = new SetProgressBarDelegate(SetProgressBar);

            string OutputFileName = "";
            string ThisLine;
            string OutputLine;
            string PreviousTickPrice = "";
            string CurrentTickPrice = "";
            int CurrentLineDate;
            int StartDate;
            int EndDate;
            int TotalInputTicks = 0;
            int TotalOutputTicks = 0;
            int SamePriceTicks = 0;
            bool DateInRange = true;
            int FilesTotal = 0;
            Exception Error = null;

            StartDate = int.Parse(KTools.DateTimeToYYYYMMDD(AppSettings.StartDate));
            EndDate = int.Parse(KTools.DateTimeToYYYYMMDD(AppSettings.EndDate));

            foreach (string CurrentFile in InputFiles)
            {
                if (!Abort
                        && !(CurrentFile.Length >= 40 && CurrentFile.Substring(CurrentFile.Length - 4, 4) == ".csv"))
                {
                    string ErrorMsg = "Input filename not in a correct format";
                    StatusChanged("Error! " + ErrorMsg + ".");
                    KTools.InfoBox(ErrorMsg + ":\n" + CurrentFile, "Error");
                }
                else if (!Abort)
                {
                    StatusChanged("Working on: " + CurrentFile);
                    FilesTotal++;

                    OutputFileName = OutputFolder + @"\" + ExtractSymbolNameFromFileName(CurrentFile);
                    File.Delete(OutputFileName);

                    StreamReader MyReader = new StreamReader(CurrentFile);
                    StreamWriter MyWriter = new StreamWriter(OutputFileName, true);

                    try
                    {
                        MyWriter.WriteLine("date,time,close");

                        while (MyReader.EndOfStream != true && !Abort)
                        {
                            ThisLine = MyReader.ReadLine();

                            TotalInputTicks++;
                            if (TotalInputTicks % 10000 == 0) backgroundWorker1.ReportProgress(0);

                            if (!AppSettings.AllData)
                            {
                                DateInRange = true;
                                CurrentLineDate = int.Parse(KTools.PortionFromCSVLine(ThisLine, 2, StartFrom.End));
                                if (!(CurrentLineDate >= StartDate && CurrentLineDate <= EndDate))
                                {
                                    DateInRange = false;
                                }
                            }

                            if (DateInRange)
                            {
                                switch (AppSettings.RemoveSamePrice)
                                {
                                    case true:
                                        CurrentTickPrice = KTools.PortionFromCSVLine(ThisLine, 0, StartFrom.End);
                                        if (CurrentTickPrice != PreviousTickPrice)
                                        {
                                            PreviousTickPrice = CurrentTickPrice;
                                            OutputLine = KTools.PortionFromCSVLine(ThisLine, 2, StartFrom.End) + ",";
                                            OutputLine += KTools.PortionFromCSVLine(ThisLine, 1, StartFrom.End) + ",";
                                            OutputLine += KTools.PortionFromCSVLine(ThisLine, 0, StartFrom.End);
                                            MyWriter.WriteLine(OutputLine);
                                            TotalOutputTicks++;
                                        }
                                        else SamePriceTicks++;
                                        break;
                                    case false:
                                        OutputLine = KTools.PortionFromCSVLine(ThisLine, 2, StartFrom.End) + ",";
                                        OutputLine += KTools.PortionFromCSVLine(ThisLine, 1, StartFrom.End) + ",";
                                        OutputLine += KTools.PortionFromCSVLine(ThisLine, 0, StartFrom.End);
                                        MyWriter.WriteLine(OutputLine);
                                        TotalOutputTicks++;
                                        break;
                                }
                            }
                        }
                    }
                    catch (Exception MyEx)
                    {
                        Error = MyEx;
                        this.Invoke(DisableAbort);
                        StatusChanged("Error! " + Error.Message);
                        KTools.InfoBox(MyEx.Message, "Error!");
                    }
                    finally
                    {
                        MyReader.Dispose();
                        MyWriter.Dispose();
                    }
                }
            }
            if (Error != null)
            {
                if (OutputFileName != "") File.Delete(OutputFileName);
            }
            else if (Abort)
            {
                StatusChanged("Aborting...");
                if (OutputFileName != "") File.Delete(OutputFileName);
                KTools.InfoBox("Aborted by user!", "Aborted");
            }
            else
            {
                this.Invoke(DisableAbort);
                this.Invoke(SetProgress, 100);
                StatusChanged("Done processing.");
                decimal Percentage;
                if (TotalInputTicks > 0)
                {
                    Percentage = decimal.Round(((decimal)((double)SamePriceTicks / TotalInputTicks) * 100), 2);
                }
                else
                {
                    Percentage = 0;
                }

                string DisplayString = "Done processing.\nTotal files: " + FilesTotal.ToString()
                    + "\nTotal input ticks: " + TotalInputTicks.ToString()
                    + "\nTotal output ticks: " + TotalOutputTicks.ToString()
                    + "\nSame price ticks removed: " + SamePriceTicks.ToString() + " (" + Percentage.ToString() + "%)";
                KTools.InfoBox(DisplayString, "Done");
            }
        }

        private string ExtractSymbolNameFromFileName(string FileName)
        {
            if (FileName.Substring(FileName.Length - 38, 1) == @"\")
            {
                return FileName.Substring(FileName.Length - 37, 6);
            }
            else if (FileName.Substring(FileName.Length - 40, 1) == @"\")
            {
                return FileName.Substring(FileName.Length - 39, 6);
            }
            else return "";
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start(@"C:\Program Files\Internet Explorer\IEXPLORE.EXE", "http://kreslik.com/forums/viewtopic.php?p=1723#1723");
        }

        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            EndConversionVisuals();
        }

        private void buttonAbort_Click(object sender, EventArgs e)
        {
            Abort = true;
            this.buttonAbort.Text = "Aborting...";
            this.buttonAbort.Enabled = false;
            this.progressBar.Text = "Aborting...";
            Application.DoEvents();
        }

        private void SelectInputFolder(object sender, EventArgs e)
        {
            this.folderBrowserDialog1.SelectedPath = AppSettings.LastInputFolder;

            DialogResult DResult = this.folderBrowserDialog1.ShowDialog();
            if (DResult == DialogResult.OK)
            {
                InputFolderSelected = true;
                AppSettings.LastInputFolder = folderBrowserDialog1.SelectedPath;
                this.labelInputFolder.Text = AppSettings.LastInputFolder;
                InputFiles = Directory.GetFiles(AppSettings.LastInputFolder);
            }
        }

        private void SelectOutputFolder(object sender, EventArgs e)
        {
            this.folderBrowserDialog1.SelectedPath = AppSettings.LastOutputFolder;

            DialogResult DResult = this.folderBrowserDialog1.ShowDialog();
            if (DResult == DialogResult.OK)
            {
                OutputFolderSelected = true;
                AppSettings.LastOutputFolder = folderBrowserDialog1.SelectedPath;
                this.labelOutputFolder.Text = AppSettings.LastOutputFolder;
                OutputFolder = AppSettings.LastOutputFolder;
            }
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (this.progressBar.Value == 100)
            {
                this.progressBar.Value = 0;
            }
            this.progressBar.PerformStep();
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            AppSettings.RemoveSamePrice = this.checkBoxRemoveSame.Checked;
        }

        private void radioButtonAlldata_CheckedChanged(object sender, EventArgs e)
        {
            AppSettings.AllData = this.radioButtonAlldata.Checked;
            ChangeDataSelectionVisuals();
        }

        private void StartDate_ValueChanged(object sender, EventArgs e)
        {
            AppSettings.StartDate = StartDate.Value;
        }

        private void EndDate_ValueChanged(object sender, EventArgs e)
        {
            AppSettings.EndDate = EndDate.Value;
        }

        private void ShowTSExportCode(object sender, EventArgs e)
        {
            ShowTSexportCode TSExportForm = new ShowTSexportCode();
            TSExportForm.ShowDialog();
        }

   }
}